/*
 * MINIGUI - Harbour Win32 GUI library Demo
 *
 * Copyright 2011-2013 Grigory Filatov <gfilatov@inbox.ru>
*/

#include "minigui.ch"

*-----------------------------------------------------------------------------*
Procedure Main
*-----------------------------------------------------------------------------*

	DEFINE WINDOW Form_1 ;
		AT 0,0 ;
		WIDTH 440 + GetBorderWidth() ;
		HEIGHT 300 + GetTitleHeight() + GetBorderHeight() ;
		TITLE 'WebCam Preview Demo' ;
		MAIN ;
		NOMAXIMIZE NOSIZE ;
		ON INIT CaptureImage() ;  // capture initialization
		ON RELEASE CloseWebCam()

		@ 20,60 WEBCAM WebCam_1 ;
			WIDTH 250 HEIGHT 210 ;
			RATE 20 ;
			START

		DEFINE IMAGE Image_1
			ROW	120
			COL	280
			WIDTH   150
			HEIGHT  110
			STRETCH .T.
		END IMAGE

		DEFINE BUTTON Button_1
			ROW	10
			COL	20
			WIDTH   120
			CAPTION 'Start WebCam'
			ACTION  CreateWebCam()
		END BUTTON

		DEFINE BUTTON Button_2
			ROW	10
			COL	150
			WIDTH   120
			CAPTION 'Stop WebCam'
			ACTION  CloseWebCam()
		END BUTTON

		DEFINE BUTTON Button_3
			ROW	80
			COL	315
			WIDTH   80
			CAPTION 'Capture'
			ACTION  CaptureImage()
		END BUTTON

		DEFINE LABEL Label_1
			ROW	59
			COL	19
			WIDTH   252
			HEIGHT  212
			BORDER  .T.
		END LABEL

		DEFINE LABEL Label_2
			ROW	119
			COL	279
			WIDTH   152
			HEIGHT  112
			BORDER  .T.
		END LABEL

		ON KEY ESCAPE ACTION ThisWindow.Release

	END WINDOW

	CENTER WINDOW Form_1

	ACTIVATE WINDOW Form_1

Return

*-----------------------------------------------------------------------------*
Procedure CreateWebCam
*-----------------------------------------------------------------------------*

	If ! IsControlDefined( WebCam_1, Form_1 )

		@ 20,60 WEBCAM WebCam_1 OF Form_1 ;
			WIDTH 250 HEIGHT 210 ;
			RATE 20

		Form_1.WebCam_1.Start()

		Form_1.Button_3.Enabled := .T.

	EndIf

Return

*-----------------------------------------------------------------------------*
Procedure CloseWebCam
*-----------------------------------------------------------------------------*

	If IsControlDefined( WebCam_1, Form_1 )

		Form_1.WebCam_1.Release()

		Form_1.Button_3.Enabled := .F.

	EndIf

Return

*-----------------------------------------------------------------------------*
Procedure CaptureImage
*-----------------------------------------------------------------------------*
  Local hBitmap
  Local nWidth
  Local nHeight

  If cap_EditCopy( GetControlHandle ( 'WebCam_1', 'Form_1' ) )

     nWidth := GetProperty( "Form_1", "Image_1", "Width" )
     nHeight := GetProperty( "Form_1", "Image_1", "Height" )

     hBitmap := LoadFromClpbrd( GetFormHandle( 'Form_1' ), nWidth, nHeight )

     If !Empty( hBitmap )

        SetImageControl ( "Form_1", "Image_1", hBitmap )

        System.Clipboard := ""

     EndIf

  Else

     MsgAlert( 'Capture is failure!', 'Error' )

  EndIf

Return

#define CF_BITMAP     2
*-----------------------------------------------------------------------------*
Static Function LoadFromClpbrd( hWnd, w, h )
*-----------------------------------------------------------------------------*
  Local hBmp

  If OpenClipboard( hWnd )

     hBmp := GetClipboardData( CF_BITMAP, w, h )

     CloseClipboard()

  EndIf

Return( hBmp )

*-----------------------------------------------------------------------------*
Static Procedure SetImageControl (cFormName, cControlImageName, hBitmap)
*-----------------------------------------------------------------------------*
Local hWnd, k

k := GetControlIndex (cControlImageName, cFormName)

IF k > 0
   IF _HMG_aControlBrushHandle [k] <> 0
       DeleteObject (_HMG_aControlBrushHandle [k])
   ENDIF
   _HMG_aControlBrushHandle [k] := hBitmap

   hWnd := GetControlHandle (cControlImageName, cFormName)

   #define STM_SETIMAGE 0x0172
   #define IMAGE_BITMAP 0

   SendMessage (hWnd, STM_SETIMAGE, IMAGE_BITMAP, hBitmap)

   #undef IMAGE_BITMAP
   #undef STM_SETIMAGE
ENDIF

Return


#pragma BEGINDUMP

#include <mgdefs.h>
#include <windows.h>
#include "hbapi.h"

HBITMAP StretchBitmap( HBITMAP hbmpSrc, int New_Width, int New_Height )
{
   HBITMAP hbmpOldSrc, hbmpOldDest, hbmpNew;
   HDC     hdcSrc, hdcDest;
   BITMAP  bmp;
   POINT   Point;

   hdcSrc = CreateCompatibleDC( NULL );
   hdcDest = CreateCompatibleDC( hdcSrc );

   GetObject( hbmpSrc, sizeof( BITMAP ), &bmp );

   hbmpOldSrc = (HBITMAP) SelectObject( hdcSrc, hbmpSrc );

   hbmpNew = CreateCompatibleBitmap( hdcSrc, New_Width, New_Height );

   hbmpOldDest = (HBITMAP) SelectObject( hdcDest, hbmpNew );

   GetBrushOrgEx( hdcDest, &Point );
   SetStretchBltMode( hdcDest, HALFTONE );
   SetBrushOrgEx( hdcDest, Point.x, Point.y, NULL );

   StretchBlt( hdcDest, 0, 0, New_Width, New_Height, hdcSrc, 0, 0, bmp.bmWidth, bmp.bmHeight, SRCCOPY );

   SelectObject( hdcDest, hbmpOldDest );
   SelectObject( hdcSrc, hbmpOldSrc );

   DeleteDC( hdcDest );
   DeleteDC( hdcSrc );

   return hbmpNew;
}

HB_FUNC( CLOSECLIPBOARD )
{
   hb_retl( CloseClipboard() );
}

HB_FUNC( OPENCLIPBOARD )
{
   hb_retl( OpenClipboard( (HWND) HB_PARNL( 1 ) ) ) ;
}

HB_FUNC( GETCLIPBOARDDATA )
{
   WORD wType = hb_parni( 1 );
   HGLOBAL hMem;

   switch( wType )
   {
      case CF_TEXT:
           hMem = GetClipboardData( CF_TEXT );
           if( hMem )
           {
              hb_retc( ( char * ) GlobalLock( hMem ) );
              GlobalUnlock( hMem );
           }
           else
              hb_retc( "" );
           break;

      case CF_BITMAP:
           if( IsClipboardFormatAvailable( CF_BITMAP ) )
              HB_RETNL( ( LONG_PTR ) StretchBitmap( (HBITMAP) GetClipboardData( CF_BITMAP ), hb_parni( 2 ), hb_parni( 3 ) ) );
           else
              HB_RETNL( ( LONG_PTR ) NULL );
   }
}

#pragma ENDDUMP
